#!/usr/bin/python
# -*- coding: utf-8 -*-

from optparse import OptionParser
import sys
import time

template_plugin = """# -*- coding: utf-8 -*-
#  ${DESCRIPTION}
# 
#  Copyright (C) ${YEAR} ${AUTHOR}
#   
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#   
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#   
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330,
#  Boston, MA 02111-1307, USA.

import gedit, gtk
from gettext import gettext as _

# Menu item example, insert a new item in the Tools menu
ui_str = \"""<ui>
  <menubar name="MenuBar">
    <menu name="ToolsMenu" action="Tools">
      <placeholder name="ToolsOps_2">
        <menuitem name="${NAME_CAMEL}" action="${NAME_CAMEL}"/>
      </placeholder>
    </menu>
  </menubar>
</ui>
\"""

class ${NAME_CAMEL}PluginInstance:
	def __init__(self, plugin, window):
		self._window = window
		self._plugin = plugin
		
		# Insert menu items
		self._insert_menu()

	def stop(self):
		# Remove any installed menu items
		self._remove_menu()
		
		self._window = None
		self._plugin = None
		self._action_group = None
		
	def _insert_menu(self):
		# Get the GtkUIManager
		manager = self._window.get_ui_manager()
		
		# Create a new action group
		self._action_group = gtk.ActionGroup("${NAME_CAMEL}PluginActions")
		self._action_group.add_actions([("${NAME_CAMEL}", None, _("${NAME}"), None, _("Example menu item"), lambda a: self.on_example_menu_item_activate())])
		
		# Insert the action group
		manager.insert_action_group(self._action_group, -1)
		
		# Merge the UI
		self._ui_id = manager.add_ui_from_string(ui_str)

	def _remove_menu(self):
		# Get the GtkUIManager
		manager = self._window.get_ui_manager()
		
		# Remove the ui
		manager.remove_ui(self._ui_id)
		
		# Remove the action group
		manager.remove_action_group(self._action_group)
		
		# Make sure the manager updates
		manager.ensure_update()

	def update(self):
		# Called whenever the window has been updated (active tab
		# changed, etc.)
		return

	# Menu activate handlers
	def on_example_menu_item_activate(self):
		# Do something here when the menu item is activated
		return

class ${NAME_CAMEL}Plugin(gedit.Plugin):
	DATA_TAG = "${NAME_CAMEL}PluginInstance"
	
	def __init__(self):
		gedit.Plugin.__init__(self)

	def _get_instance(self, window):
		return window.get_data(self.DATA_TAG)
	
	def _set_instance(self, window, instance):
		window.set_data(self.DATA_TAG, instance)
	
	def activate(self, window):
		self._set_instance(window, ${NAME_CAMEL}PluginInstance(self, window))
	
	def deactivate(self, window):
		self._get_instance(window).stop()
		self._set_instance(window, None)
		
	def update_ui(self, window):
		self._get_instance(window).update()
"""

template_desktop = """[Gedit Plugin]
Loader=python
Module=${NAME_LOWER}
IAge=2
Name=${NAME}
Description=${DESCRIPTION}
Authors=${AUTHOR}
Copyright=Copyright © ${YEAR} ${AUTHOR}
Website=http://www.gedit.org
"""

def parse_options():
	parser = OptionParser()
	parser.add_option("-n", "--name", action="store", type="string",
			dest="name", help="the name of the plugin (e.g. Color picker)", 
			metavar="NAME")
	parser.add_option("-a", "--author", action="store", type="string",
			dest="author", help="the name of the person who will write the plugin (usually you)", 
			metavar="AUTHOR", default="<author>")
	parser.add_option("-d", "--description", action="store", type="string",
			dest="description", help="the description of the plugin (e.g. Pick a color from a dialog and insert its hexadecimal representation.)", 
			metavar="DESCRIPTION", default="<description>")

	return parser.parse_args()

(options, args) = parse_options()

if not options.name:
	print "You need to specify the name of the plugin"
	sys.exit(1)

name = options.name.lower()
lower = str.join('', name.split(' '))
camel = ''

for part in name.split(' '):
	camel += part.capitalize()

subst = {	'NAME': name.capitalize(), 
		'NAME_LOWER': lower,
		'NAME_CAMEL': camel,
		'DESCRIPTION': options.description,
		'AUTHOR': options.author,
		'YEAR': time.strftime("%Y")}

templates = {	lower + '.py': template_plugin,
		lower + '.gedit-plugin': template_desktop}

for filename in templates:
	template = templates[filename]

	print 'Generating', filename
	
	for target in subst:
		replace = subst[target]
		template = template.replace('${' + target + '}', replace)

	f = open(filename, 'w')
	f.write(template)
	f.close()	
